<?php

namespace Drupal\entity_workflow_content\Plugin\WorkflowType;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeBase;
use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\path_alias\Entity\PathAlias;
use Drupal\workflows\TransitionInterface;
use Drupal\workflows\WorkflowInterface;

/**
 * @WorkflowType(
 *   id = "entity_workflow_content",
 *   label = @Translation("Content"),
 *   required_states = {
 *     "draft",
 *     "review",
 *     "approved",
 *   },
 *   forms = {
 *     "configure" = "\Drupal\entity_workflow\Form\EntityWorkflowConfigureForm",
 *     "transition" = "\Drupal\entity_workflow\Form\EntityWorkflowTransitionForm",
 *   },
 * )
 */
class EntityWorkflowContent extends EntityWorkflowTypeBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $default_configuration = [
      'entity_types' => [
        'node' => [],
      ],
      'default_state' => 'draft',
      'default_transition' => 'new_draft',
      'states' => [
        'draft' => [
          'label' => t('Draft'),
          'weight' => 0,
        ],
        'review' => [
          'label' => t('Ready for Review'),
          'weight' => 30,
        ],
        'approved' => [
          'label' => t('Approved'),
          'weight' => 100,
        ],
      ],
      'transitions' => [
        'new_draft' => [
          'label' => t('New draft'),
          'from' => [],
          'to' => 'draft',
          'weight' => 0,
        ],
        'submit_for_review' => [
          'label' => t('Submit for Review'),
          'from' => ['draft'],
          'to' => 'review',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => 10,
        ],
        'approve' => [
          'label' => t('Approve'),
          'from' => ['review'],
          'to' => 'approved',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => 100,
        ],
        'back_to_draft' => [
          'label' => t('Back to Draft'),
          'from' => ['review', 'approved'],
          'to' => 'draft',
          'transition_log' => TRUE,
          'action_link' => TRUE,
          'weight' => -100,
        ],
        // Special "skip to the end" transition.
        'approve_immediately' => [
          'label' => t('Approve immediately'),
          'from' => ['draft', 'review'],
          'to' => 'approved',
          'transition_log' => TRUE,
          'transition_log_required' => TRUE,
          'action_link' => TRUE,
          'weight' => 100,
          // This only shows up if 'approve' doesn't.
          'additional_access_callbacks' => ['entity_workflow_content_can_not_approve'],
        ],
      ],
      // This content workflow implementation requires an active workspace for
      // any entity operation (add, edit, etc.). The exclusion list below allows
      // specified entity types or only a subset of their bundles to bypass the
      // workspace requirement, so entity operation can also happen in 'Live'.
      // Exclusions are arrays keyed by entity type IDs, with values either an
      // empty array, which means all bundles are excluded, or a specific list
      // of bundle names to be excluded.
      'exclude_from_workspace_requirement' => [],
    ] + parent::defaultConfiguration();

    return $default_configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function getPermissions() {
    $perms = [];

    // Add generic permission to transition to any state.
    $perms['use all content transitions'] = [
      'title' => $this->t('%workflow workflow: Use all transitions', [
        '%workflow' => $this->label(),
      ]),
      'description' => $this->t('Allow the user to transition from any state to any state, as defined in the %workflow workflow.', [
        '%workflow' => $this->label(),
      ]),
    ];

    foreach ($this->getTransitions() as $transition) {
      $perms['use content transition ' . $transition->id()] = [
        'title' => $this->t('%workflow workflow: Use %transition transition', [
          '%workflow' => $this->label(),
          '%transition' => $transition->label(),
        ]),
        'description' => $this->t('Allow the user to use the %transition transition, as defined in the %workflow workflow.', [
          '%workflow' => $this->label(),
          '%transition' => $transition->label(),
        ]),
      ];
    }

    return $perms;
  }

  /**
   * {@inheritdoc}
   */
  public function onPostTransition(TransitionInterface $transition, WorkflowInterface $workflow, ContentEntityInterface $entity, $field_name, array $context = []) {
    parent::onPostTransition($transition, $workflow, $entity, $field_name, $context);

    // Improve the UX for some entity types which have related entities that are
    // usually managed in the same screen, for example path aliases, menu links
    // for nodes, etc.
    $related_entities = [];

    // Add the entity's path alias.
    foreach ($entity->getFieldDefinitions() as $field => $field_definition) {
      if ($field_definition->getType() === 'path' && ($path_alias_id = $entity->get($field)->get(0)->pid)) {
        $related_entities[] = PathAlias::load($path_alias_id);
      }
    }

    // Add the menu link for nodes.
    if ($entity->getEntityTypeId() === 'node') {
      $menu_defaults = menu_ui_get_menu_link_defaults($entity);
      if ($menu_link_id = $menu_defaults['entity_id']) {
        $related_entities[] = MenuLinkContent::load($menu_link_id);
      }
    }

    foreach ($related_entities as $entity) {
      entity_workflow_system_transition($entity, $workflow->id(), $transition->to()->id(), '', $context);
    }

    // Invalidate the cache tags of the active workspace to ensure that its
    // local actions get updated. An example why this is needed is when all the
    // workspace content get to the 'Approved' state, the 'Publish' local action
    // needs to become available for the workspace.
    if ($workspace = $this->workspaceManager->getActiveWorkspace()) {
      Cache::invalidateTags($workspace->getCacheTagsToInvalidate());
    }
  }

}
